*************************************************************************************
*														*
*	A recreation of the 'Galaxy Invader 1000' game for the EASy68K simulator	*
*	V1.01													*
*														*
*	The aim of the game is to protect the base from attacking invaders and		*
*	score 1000 points. Once 1000 points is reached or exceeded the game ends	*
*	and the player has won. To this end the player has a limited store of 250	*
*	missiles and three missile launchers. An extra launcher and another 50		*
*	missiles are awarded once a score of 700 points is reached. When only 30	*
*	missiles are left the game will sound a warning and the game score will		*
*	flash. Once the missiles run out the game ends as a draw.				*
*														*
*	Up to three missiles may be in flight at any time but only one in each		*
*	column.												*
*														*
*	Keys:													*
*														*
*	[Z] moves the player to the left column, [C] moves the player to the right	*
*	column. If neither or both keys are pressed the player will be in the middle	*
*	column.												*
*														*
*	[M] fires a missile. When the game is over and the display is flashing the	*
*	keys [1], [2] and [3] set the game level and [SPACE] starts a new game.		*
*														*
*	[S] toggles the sound on/off.									*
*														*
*	The playfield is a 16 colour bitmap image loaded from the file invader.bmp	*
*														*
*	Changes:												*
*														*
*	V1.01 - Changed the initial missile count from 300 to 250, added the bonus	*
*		  50 missiles to the extra life at 700 and made the low missiles		*
*		  warning sound coincide with the score flash.					*
*														*
*	This game requires EASy68K 3.7.10 beta or later.					*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	 http://www.themotionstore.com/leeedavison/index.html					*
*														*
*	.. or ..												*
*														*
*	 http://members.lycos.co.uk/leeedavison/index.html					*
*														*
*	mail : leeedavison@lycos.co.uk								*
*														*
*************************************************************************************

*************************************************************************************
*
* program constants

* EASy68K wants 32 bit colour values in the form $00,B,G,R

red_on	EQU	$000000F2			* the colour when a red segment is on
green_on	EQU	$00F2F200			* the colour when a green segment is on
colour_off	EQU	$00323232			* the colour mask when a segment is off


*************************************************************************************
*

	ORG	$1000

start
	BSR		initialise			* go setup game stuff
	BSR		load_image			* go load the game background image

* main loop. everything is done in this loop, the overall speed of the loop is set by
* waiting for the remainder of a fixed time to pass at the end of the loop

loop
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	MOVE.l	d1,-(sp)			* push the time on the stack

	MOVE.l	#red_on,d1			* red segment on colour
	MOVE.l	#green_on,d2		* green segment on colour

	MOVEQ		#8,d0				* set count mask
	AND.w		counter(a3),d0		* mask the count
	OR.w		playmode(a3),d0		* or with the playing flag
	BNE.s		save_colours		* if set go save the colours

	MOVE.l	#colour_off,d3		* the colour mask when a segment is off
	AND.l		d3,d1				* mask the red segment colour
	AND.l		d3,d2				* mask the green segment colour
save_colours
	MOVE.l	d1,segment_red(a3)	* save the red segment colour
	MOVE.l	d2,segment_green(a3)	* save the green segment colour

	BSR		do_keys			* handle the keys

	BSR		display_sound		* draw the sound status
	BSR		display_level		* draw the level
	BSR		display_score		* draw the score
	BSR		display_lives		* draw the lives
	BSR		display_saucer		* draw the saucer
	BSR		display_missile		* draw the player missile
	BSR		display_invaders		* draw invaders
	BSR		display_fire		* draw invaders missiles
	BSR		display_player		* draw the player
	BSR		display_explosions	* draw explosions

	BSR		handle_explosions		* handle any explosions
	BSR		spawn_invaders		* spawn new invaders if none left

	MOVE.b	#94,d0			* copy screen buffer to main
	TRAP		#15

	TST.w		playmode(a3)		* test the playing flag
	BEQ.s		not_playing			* if not playing skip moving stuff

	BSR		player_miss			* handle the player's missile
	BSR		invaders_missiles		* handle the invaders' missiles
	BSR		move_invaders		* move the invaders
	BSR		move_saucer			* move the saucer

not_playing
	BSR		gen_prng			* generate a new random number
	ADDQ.w	#1,counter(a3)		* increment the flash rate counter

* now wait for a while. this limits the loop to ten redraws a second without which
* this could run way to fast if the machine was capable

main_end
	MOVE.l	(sp)+,d7			* restore time in 1/100 ths from stack
wait_100ms
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	SUB.l		d7,d1				* subtract previous time from current time
	CMP.b		#$09,d1			* compare with 9/100ths
	BMI.s		wait_100ms			* loop if time not up yet

* if you want to see how fast it can go comment out the BMI.s above

	BRA		loop				* loop forever


*************************************************************************************
*
* handle the player's missile. check for hits with invaders or the saucer, move the
* missiles, check again for invader hits.
*
* if the fire flag is set check if player fire is allowed, allow fire if so and
* decrement the missiles remaining count. play a warning sound if missile count is
* only seventy.

saucer_pattern
	dc.l	$80000000				* saucer in position 1
	dc.l	$00800000				* saucer in position 2
	dc.l	$00008000				* saucer in position 3

player_miss
	MOVE.l	player_missile(a3),d4	* get the player missile segment pattern
	BEQ.s		allow_fire			* if no missile go directly to the fire test

* there is a missile in flight so test for a hit on an invader

	BSR		hit_invader			* go test for an invader hit
	TST.l		d4				* test the returned player missile pattern
	BEQ.s		save_fire			* if no missiles go save the missile pattern

	MOVE.w	saucer_position(a3),d0	* get the saucer position
	BEQ.s		no_hit_saucer		* if no saucer skip the hit test

	SUBQ.w	#1,d0				* make the saucer position 0 to 2
	ASL.w		#2,d0				* * 4 bytes per longword
	MOVE.l	saucer_pattern(pc,d0.w),d0
							* get the saucer segment pattern
	AND.l		d4,d0				* and with missile pattern
	BEQ.s		no_hit_saucer		* if no hit skip explosion set

	OR.l		d0,explosions(a3)		* OR with the invader explosions pattern
	EOR.l		d0,d4				* clear the missile bit
	MOVEQ		#10,d1			* set the saucer score
	BSR		add_score			* add it to the player's score
	MOVE.w	#0,saucer_position(a3)	* clear the saucer's position
	MOVE.w	#$10,sau_move_count(a3)	* reset the saucer's move count
	MOVEQ		#4,d1				* index 4, hit saucer
	BSR		play_sample			* play the sample
no_hit_saucer
	SUBQ.w	#1,player_mis_count(a3)	* decrement the player missile move count
	BPL.s		p_not_counted		* if positive skip the reset and the move

	MOVE.w	#1,player_mis_count(a3)	* reset the player missile move count
	ADD.l		d4,d4				* move the missile up one
p_not_counted
	AND.l		#$FCFCFC00,d4		* mask non missile bits
	BEQ.s		save_fire			* if no missiles go save the missile pattern

	BSR		hit_invader			* go test for an invader hit
save_fire
	MOVE.l	d4,player_missile(a3)	* save the player missile segment pattern
allow_fire
	BNE.s		no_end			* if missile in flight skip game end

	TST.w		missiles(a3)		* test the missiles count
	BNE.s		no_end			* if missiles skip the game end

	MOVEQ		#11,d1			* index 11, game drawn
	BSR		play_sample			* play the sample
	MOVE.w	#0,playmode(a3)		* clear the play flag
no_end
	TST.w		fire(a3)			* test the fire flag
	BEQ.s		exit_player_miss		* if not set skip fire

	MOVEQ		#4,d0				* new missile pattern
	MOVE.w	player_position(a3),d1	* get the player position
	SUBQ.w	#1,d1				* make 0 to 2
	TST.b		player_missile(a3,d1.w)	* test if a missile in this column
	BNE.s		no_fire			* if missile skip the player firing

	TST.w		missiles(a3)		* test the missiles count
	BEQ.s		no_fire			* if none left skip player fire

	MOVE.b	d0,player_missile(a3,d1.w)
							* save the new missile
	SUBQ.w	#1,missiles(a3)		* decrement the missile count
	MOVEQ		#6,d1				* index 6, player fire
	BSR		play_sample			* play the sample
	CMPI.w	#30,missiles(a3)		* compare the remainimg missiles count with 30
	BNE.s		no_fire			* if not 30 skip the sound play

	MOVEQ		#9,d1				* index 9, low ammo
	BSR		play_sample			* play the sample
no_fire
	MOVE.w	#0,fire(a3)			* clear the fire flag
exit_player_miss
	RTS


*************************************************************************************
*
* test for an invader hit. test each invader in turn, play a sound if hit.

hit_invader
	MOVE.l	invader_1(a3),d0		* get invader_1's segment pattern
	BSR.s		hit_invader_1		* go test invader 1

	MOVE.l	invader_2(a3),d0		* get invader_2's segment pattern
hit_invader_1
	AND.l		d4,d0				* test for hit
	BEQ.s		no_hit_invader		* if no hit just exit

* there was a hit so generate the explosion pattern

	OR.l		d0,explosions(a3)		* OR into the explosions segment pattern
	MOVE.l	d0,d1				* copy the hit pattern

	SWAP		d0				* possibly move bit to low word
	OR.w		d1,d0				* copy bit if in low word
	MOVE.w	d0,-(sp)			* push the word
	OR.b		(sp)+,d0			* shift word and OR into low byte

	NOT.l		d1				* invert the pattern
	AND.l		d1,d4				* mask the player missile pattern
	AND.l		d1,invader_1(a3)		* mask invader_1's segment pattern
	AND.l		d1,invader_2(a3)		* mask invader_2's segment pattern
score_hit
	MOVEQ		#5,d1				* set the score for the top invader row
score_loop
	ADD.b		d0,d0				* shift the explosion bit
	BMI.s		add_invader_score		* if this row go add the score

	SUBQ.w	#1,d1				* else decrement the score
	BRA.s		score_loop			* and loop to test the next row

add_invader_score
	BSR		add_score			* add the invader value to the score
	MOVEQ		#5,d1				* index 5, hit invader
	BRA		play_sample			* play the sample and return

no_hit_invader
	RTS


*************************************************************************************
*
* handle the invaders' missiles. test each in turn against the player position

invaders_missiles
	MOVE.w	player_position(a3),d3	* get the player position
	SUBQ.w	#1,d3				* make the player position 0 to 2
	ASL.w		#2,d3				* * 4 bytes per longword
	MOVE.l	player_pattern(pc,d3.w),d3
							* get the player segment pattern
	LEA		invader_1(a3),a1		* get the pointer to invader_1
	BSR.s		invader_miss		* handle an invaders' missile
	LEA		invader_2(a3),a1		* get the pointer to invader_2
*##	BSR.s		invader_miss		* handle an invaders' missile
*##	RTS


*************************************************************************************
*
* handle an invaders' missile

inv_mis_cnt		EQU	3			* invader's missile move count

invader_miss
	MOVE.l	invader_missile(a1),d4	* get the invader missile segment pattern
	BEQ.s		invader_fire		* if no missile go directly to the fire test

	SUBQ.w	#1,inv_mis_count(a1)	* decrement the invader missile move count
	BPL.s		i_not_counted		* if positive skip the reset and the move

	MOVE.w	#inv_mis_cnt,inv_mis_count(a1)
							* reset the invader missile move count
	LSR.l		#1,d4				* move the missile down one
i_not_counted
	AND.l		#$7E7E7E00,d4		* mask non missile bits
	MOVE.l	d4,invader_missile(a1)	* save the invader missile segment pattern

	AND.l		d3,d4				* and with the player segment pattern
	BEQ.s		no_hit_player		* if no hit skip explosion set

	OR.l		d3,explosions(a3)		* OR with the previous explosions pattern
	EOR.l		d3,invader_missile(a1)	* clear the invader missile segment pattern
	MOVE.w	lives(a3),d1		* get the lives remaining
	SUBQ.w	#1,lives(a3)		* decrement the player lives count
	BPL.s		no_end_game			* if not all gone just continue

	MOVE.w	#0,playmode(a3)		* else clear the play flag
no_end_game
	BRA		play_sample			* play the hit sample and return
		
no_hit_player
	RTS

player_pattern
	dc.l	$02000000				* player in position 1
	dc.l	$00020000				* player in position 2
	dc.l	$00000200				* player in position 3

invader_fire
	MOVE.w	inv_fire_count(a1),d0	* get the invader's fire count
	BEQ.s		allow_inv_fire		* if counted allow invader fire

	SUBQ.w	#1,d0				* else decrement the count
	MOVE.w	d0,inv_fire_count(a1)	* save the invader's fire count
	RTS

allow_inv_fire
	MOVE.l	(a1),invader_missile(a1)
							* copy the invader's position to the missile
	MOVE.w	level(a3),d1		* get the level
	ADD.w		d1,d1				* * 2 bytes per word
	LEA		move_count(pc),a0
	MOVE.w	(a0,d1.w),d1		* get the move count for this level
	MOVE.w	d1,inv_fire_count(a1)	* save the invader's fire count
	RTS


*************************************************************************************
*
* handle the explosions. explosions only last for one go round the main loop so erase
* any that were drawn this time round

handle_explosions
	TST.w		playmode(a3)		* test the playing flag
	BEQ.s		exit_handle_explosions	* if not playing just exit

	MOVEQ		#0,d0				* clear longword
	MOVE.l	d0,explosions(a3)		* clear the explosions segment pattern
exit_handle_explosions
	RTS


*************************************************************************************
*
* move the saucer and play the saucer sound if the saucer is visible

move_saucer
	MOVE.w	sau_move_count(a3),d1	* get the saucer move count
	SUBQ.w	#1,d1				* decrement the saucer move count
	BPL.s		save_move_saucer		* if not counted out go save the count

	MOVEQ		#$10,d1			* set the saucer move count
	MOVE.w	d1,d0				* copy the count
	BSR		get_prng			* get a random number based on the count
	TST.w		d0				* test the result
	BNE.s		do_saucer_noise		* if not zero don't move this time

	BSR		gen_prng			* generate a random number
	MOVE.w	saucer_position(a3),d0	* get the saucer's position
	ADDQ.w	#1,d0				* default to move right
	BTST		#0,PRNlword(a3)		* test a random bit
	BNE.s		not_left			* if set skip the move left

	SUBQ.w	#2,d0				* else move left
not_left
	ANDI.w	#3,d0				* mask to legal positions
	MOVE.w	d0,saucer_position(a3)	* save the saucer's new position
save_move_saucer
	MOVE.w	d1,sau_move_count(a3)	* save the saucer's move count
do_saucer_noise
	TST.w		saucer_position(a3)	* test the saucer's position
	BEQ.s		exit_move_saucer		* if no saucer don't play the sound

	MOVEQ		#7,d1				* index 7, saucer
	BRA		play_sample			* play the sample and return

exit_move_saucer
	RTS


*************************************************************************************
*
* spawn a pair of invaders. yes this code can get stuck in a loop if the spawn_invader
* routine keeps returning the same value for the first invader and all subsequent
* attempts to find a different position for the second invader but that is unlikely
* enough to not be a problem.

spawn_invaders
	MOVE.l	invader_1(a3),d0		* get invader 1
	OR.l		invader_2(a3),d0		* OR with invader 2
	OR.l		explosions(a3),d0		* OR with the explosions
	BNE.s		exit_spawn_invaders	* if invaders or explosions just exit

	BSR		spawn_invader		* spawn an invader
	MOVE.l	d1,invader_1(a3)		* save the result to invader 1
spawn_invaders_loop
	BSR		spawn_invader		* spawn an invader
	MOVE.l	d1,invader_2(a3)		* save the result to invader 2
	AND.l		invader_1(a3),d1		* and with invader 1
	BNE.s		spawn_invaders_loop	* if they're the same go try again

	MOVE.w	level(a3),d1		* get the level
	ADD.w		d1,d1				* * 2 bytes per word
	MOVE.w	move_count(pc,d1.w),d1	* get the move count for this level
	LEA		invader_1(a3),a1		* get the pointer to invader_1
	LEA		invader_2(a3),a2		* get the pointer to invader_2
	MOVE.w	d1,inv_move_count(a1)	* save invader_1's move count
	MOVE.w	d1,inv_move_count(a2)	* save invader_2's move count
	MOVE.w	d1,inv_fire_count(a1)	* save invader_1's fire count
	MOVE.w	d1,inv_fire_count(a2)	* save invader_2's fire count
exit_spawn_invaders
	RTS


*************************************************************************************
*
* spawn an invader in rows 5 and 4 for level one, rows 5, 4 and 3 for level two and
* rows 5, 4, 3 and 2 for level three. returns the pattern in d1.l

spawn_invader
	MOVE.w	level(a3),d0		* get the level
	ADDQ.w	#1,d0				* + 1 to make 2, 3 or 4
	MULU.w	#12,d0			* * bytes per row pattern
	BSR		get_prng			* get a random number based on the level
	MOVEQ		#-4,d1			* set longword mask
	AND.w		d1,d0				* mask offset to longword
	MOVE.l	invader_longword(pc,d0.w),d1
							* get the invader longword
	RTS

invader_longword
	dc.l	$40000000,$00400000,$00004000	* row 5 patterns
	dc.l	$20000000,$00200000,$00002000	* row 4 patterns
	dc.l	$10000000,$00100000,$00001000	* row 3 patterns
	dc.l	$08000000,$00080000,$00000800	* row 2 patterns


*************************************************************************************
*
* move the invaders

move_count	EQU	*-2				* no zeroth element
	dc.w	$0010					* move count for level 1
	dc.w	$000C					* move count for level 2
	dc.w	$0008					* move count for level 3

move_invaders
	LEA		invader_1(a3),a1		* get the pointer to invader_1
	LEA		invader_2(a3),a2		* get the pointer to invader_2
	BSR.s		move_invader		* go move invader_1
	EXG		a1,a2				* exchange the invader pointers
*##	BSR.s		move_invader		* go move invader_2
*##	RTS


*************************************************************************************
*
* move an invader

move_invader
	MOVE.w	inv_move_count(a1),d1	* get the invader move count
	SUBQ.w	#1,d1				* decrement the invader move count
	BPL.s		save_invader_move		* if not counted out go save the count

	MOVE.w	level(a3),d1		* get the level
	ADD.w		d1,d1				* * 2 bytes per word
	MOVE.w	move_count(pc,d1.w),d1	* get the move count for this level
	MOVE.w	d1,d0				* copy the count
	BSR		get_prng			* get a random number based on the count
	TST.w		d0				* test the result
	BNE.s		exit_move_invader		* if not zero don't move this time

	MOVEQ		#4,d0				* set the number of ways the invader can move
	BSR		get_prng			* get a random number from 0 to ways - 1
	ADD.w		d0,d0				* * 2 bytes per shift count
	MOVE.w	shift_count(pc,d0.w),d0	* get the shift count for the move
	MOVE.l	(a1),d2			* get the invader's position
	ROR.l		d0,d2				* shift it
	TST.b		d2				* make sure the invader isn't out of bounds
	BEQ.s		move_was_ok			* if the move was ok skip the correction

	SWAP		d2				* else move the invader to the middle column
move_was_ok
	CMP.l		(a2),d2			* compare the invader's positions
	BEQ.s		exit_move_invader		* if the positions are the same forget the move

	MOVE.l	d2,(a1)			* save the new invader position
	AND.l		#$02020200,d2		* mask for bottom row
	BEQ.s		save_invader_move		* if not the bottom row skip end game

	MOVE.w	#0,playmode(a3)		* end the game
save_invader_move	
	MOVE.w	d1,inv_move_count(a1)	* save the invader's move count
exit_move_invader	
	RTS

* the shift count works to move the invader as follows. an ROR count of $0001 will move
* an invader down one position. an ROR count of $0008 will move an invader to the next
* column on the right and an ROR count of $0018 will move an invader to the next column
* on the left. a move right from the rightmost column or a move left from the leftmost
* column will place the invader in the low byte of the longword which is out of bounds.
* this can easily be tested for and corrected by swapping the words in the longword to
* move the invader into the middle column

* so that the invader moves down as often as it moves left or right there are two
* down counts

shift_count
	dc.w	$0001					* move down
	dc.w	$0001					* move down
	dc.w	$0008					* move right
	dc.w	$0018					* move left


*************************************************************************************
*
* add d1.w to the player's score. check for the bonus life at 700 and game end at 1000

add_score
	MOVE.w	score(a3),d0		* get the old score
	ADD.w		d0,d1				* calculate the new score
	MOVE.w	d1,score(a3)		* save the new score
	CMPI.w	#700,d1			* compare the new score with extra life score
	BLT.s		exit_add_score		* if less don't add a new life

	CMPI.w	#700,d0			* compare the old score with extra life score
	BGE.s		no_launcher			* if >= don't add a new life

	ADDQ.w	#1,lives(a3)		* else increment the lives
	ADD.w		#50,missiles(a3)		* and the missile count
	MOVEQ		#8,d1				* index 8, extra life
	BSR		play_sample			* play the sample
no_launcher
	CMPI.w	#1000,d1			* compare the new score with game won score
	BLT.s		exit_add_score		* if less don't end the game

	MOVE.w	#1000,score(a3)		* set the score to 1000
	MOVE.w	#0,playmode(a3)		* clear the play flag
	MOVEQ		#10,d1			* index 10, game won
	BRA		play_sample			* play the sample and return

exit_add_score
	RTS


*************************************************************************************
*
* setup stuff

initialise
	MOVEQ		#50,d0			* close all files
	TRAP		#15

	MOVEQ		#17,d1			* enable double buffering
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	LEA		variables(pc),a3		* set the base pointer to the variables

	BSR		prng_init			* initialise the PRNG
	BSR		sound_init			* load the sound effects
	MOVE.w	#-1,play_sound(a3)	* set the play sound flag

	MOVE.w	#0,playmode(a3)		* clear the play flag
	MOVE.w	#1,level(a3)		* set the game level
	MOVE.w	#2,player_position(a3)	* set the player position
	BRA.s		reset_game			* go reset the game


*************************************************************************************
*
* start a new game

start_game
	MOVE.w	#-1,playmode(a3)		* set the play flag
reset_game
	MOVEQ		#0,d0				* clear longword
	MOVE.w	d0,score(a3)		* clear the score
	MOVE.w	d0,fire(a3)			* clear the fire flag
	MOVE.w	#2,lives(a3)		* set the player's lives count
	MOVE.w	#250,missiles(a3)		* set the player's missiles count
	MOVE.l	d0,player_missile(a3)	* clear the player missile segment pattern
	MOVE.l	d0,explosions(a3)		* clear the explosions segment pattern
	LEA		invader_1(a3),a1		* get the pointer to invader_1
	LEA		invader_2(a3),a2		* get the pointer to invader_2
	MOVE.l	d0,(a1)			* clear invader_1's segment pattern
	MOVE.l	d0,(a2)			* clear invader_2's segment pattern
	MOVE.l	d0,invader_missile(a1)	* clear invader_1's missile segment pattern
	MOVE.l	d0,invader_missile(a2)	* clear invader_2's missile segment pattern
	MOVE.w	d0,saucer_position(a3)	* clear the saucer position

	RTS


*************************************************************************************

	INCLUDE	background.x68
							* bitmap load routines
	INCLUDE	keyboard.x68
							* keyboard routines
	INCLUDE	display.x68
							* display routines
	INCLUDE	sounds.x68
							* sound routines
	INCLUDE	random.x68
							* prng routines


*************************************************************************************
*
* variables used

variables						* points to variables base

	OFFSET	0				* going to use relative addressing

score
	ds.w	1					* the score
score_disp
	ds.l	1					* space for score digits

counter
	ds.w	1					* counter for flash rates
playmode
	ds.w	1					* playing/game over flag
level
	ds.w	1					* game level. 1, 2 or 3

segment_red
	ds.l	1					* the colour when a red segment is on
segment_green
	ds.l	1					* the colour when a green segment is on

s_key
	ds.w	1					* s key last status
lives
	ds.w	1					* lives remainimg
missiles
	ds.w	1					* missiles remainimg
fire
	ds.w	1					* player fired flag
player_position
	ds.w	1					* player position
player_missile
	ds.l	1					* player missile segment position
player_mis_count
	ds.w	1					* player missile movement count

explosions
	ds.l	1					* explosions - player, saucer and invader

invader_1
	ds.l	1					* invader_1's segment pattern
inv_move_count	EQU				*-invader_1
	ds.w	1					* invader_1's move count
invader_missile	EQU				*-invader_1
	ds.l	1					* invader_1's missile segment pattern
inv_mis_count	EQU				*-invader_1
	ds.w	1					* invader_1's missile movement count
inv_fire_count	EQU				*-invader_1
	ds.w	1					* invader_1's fire count

invader_2
	ds.l	1					* invader_2's segment pattern
	ds.w	1					* invader_2's move count
	ds.l	1					* invader_2's missile segment pattern
	ds.w	1					* invader_2's missile movement count
	ds.w	1					* invader_2's fire count

saucer_position
	ds.w	1					* saucer position
sau_move_count
	ds.w	1					* saucer's move count

PRNlword
	ds.l	1					* pseudo random longword

im_x
	ds.w	1					* image x origin
im_y
	ds.w	1					* image y origin

im_width
	ds.l	1					* image width
im_height
	ds.l	1					* image height
im_depth
	ds.w	1					* image depth in bits per pixel
im_palette
	ds.l	16					* image palette space if needed

play_sound
	ds.w	1					* play sound flag

im_file
							* the rest of the memory is the file buffer

*************************************************************************************

	END	start


*************************************************************************************
